from joblib import Parallel, delayed
from features import *

import random
import math
from scipy.optimize import fsolve
import sys
import os
import copy
from bfi import *

random.seed(200)

def get_filepaths(directory):
    """
    This function will generate the file names in a directory 
    tree by walking the tree either top-down or bottom-up. For each 
    directory in the tree rooted at directory top (including top itself), 
    it yields a 3-tuple (dirpath, dirnames, filenames).
    """
    file_paths = []  # List which will store all of the full filepaths.

    # Walk the tree.
    for root, directories, files in os.walk(directory):
        for filename in files:
            # Join the two strings in order to form the full filepath.
            filepath = os.path.join(root, filename)
            file_paths.append(filepath)  # Add it to the list.

    return file_paths  # Self-explanatory.

# Run the above function and store its results in a variable.   
full_file_paths = get_filepaths(".")

#print([ x for x in full_file_paths if x.split("/")[-1][:2] =="sp" ])


paths = [ x for x in full_file_paths if x.split("/")[-1][:2] =="sp" ]#[20:25]

random.shuffle(paths)

print(len(paths))
paths = paths#[:100] 
def benjamin_feir_index(depth, frequency, bandwidth, total_energy):
	def equation(x):
		return 9.81 * x * math.tanh(x * depth) - (2 * math.pi * frequency)**2
	kp = fsolve(equation, 0 ) #peak wavenumber
	#print(kp)
	#replace it by a a large number
	#later replace it
	if frequency ==0:
		#frequency = 0.00001
		return 0
	omega = bandwidth / ( 2 * math.pi * frequency )
	bfi = math.sqrt(2) * kp * 4 * math.sqrt(total_energy) / (4 * omega)
	return bfi

def process_data(mylist):
	#print(mylist)
	#depth of water in the ocean, is approx like that since its the ocean
	#depth = 100000
	station_name = " ".join([ val for val in mylist[0].split(" ") if val.strip()!="" ][2:6] )
	depth = [ x for x in mylist[3].split(" ") if x.strip()!=''][2]
	#print(depth)
	if depth == 'N/A':
		depth = -20000
	#print(depth)
	other_vars = [x for x in mylist[5].split(" ") if x.strip()!=""]
	#print(other_vars)
	hs = float(other_vars[1])
	tp = float(other_vars[3])
		
	#dp = float(other_vars[5])
	ta = float(other_vars[7])
	spec_data = [ x for x in mylist[10:] if x!='' and x!= [] ]
	#print(spec_data)
	max_energy = 0
	fourier_vals = []
	max_freq = 0
	max_bandwidth = 0
	total_energy = 0
	dmean_max = 0
	bfi_ = 0
	spectral_data = []	
	energies = []
	if spec_data == [] :
		return [0]
	outs = []
	frequencies = []

	bandwidths = []
	for spec in spec_data:

		dat = [x for x in spec.split(" ") if x.strip()!="" ]
		#print(dat)
		try:
			#elgar and seymour 1985 effects of lack of stationarity in deep water waves
			#if float(dat[0] ) < 0.06 or float(dat[0] ) > 0.14:
			#	continue	
			frequencies.append( float(dat[0] )  )
			energy = float(dat[2])
			#from energy, peak direction and spectra
			spectral_data.append([ float(x) for x in dat[2: 8] ] )
			#except Exception as e:
			#	print(e)
			#if float(dat[0] ) >= 0.06 and  float(dat[0] ) <= 0.14:
			outs.append([ float(x) * energy * float(dat[1] ) for x in dat[4: 8] ] )
			energies.append(energy)
			bandwidths.append( float(dat[1] ) )
			
			#total_energy.append(energy * float(dat[0]) )
			total_energy += float(dat[1] ) * energy 
			#if energy > max_energy:
				#max_energy = energy
				#max_freq = float( dat[0] )
				#max_bandwidth = float( dat[1] )
				#dmean_max = float( dat[3] )
				#fourier_vals = [float(x) for x in dat[4: 8]]
				#fourier_vals.append(float( dat[1] ) / float( dat[0] ) )
				#fourier_vals.append( float(dat[2]) )
					
				#now lets calculate bfi
				#bfi_ = bfi(fourier_vals[0], fourier_vals[1], fourier_vals[2], fourier_vals[3] )
				#print("bfi is " + str(bfi_) )
				#bfi = benjamin_feir_index(depth, max_freq, max_bandwidth, total_energy )
		except Exception as e:
			print(e)
			return [0]
			#return [0, 0, 0, 0, 0,0,0]
	
	if len( outs ) == 0:
		return [0]
	#lets do integration of fourier vals with the respective frequencies
	
	integrals = outs
	#print(frequencies)
	#print(spectral_data)
	
	#print(spectral_data)
	tot_energy = total_energy
	#print(tot_energy)
	cov_corr = covariance_and_correlation(spectral_data)
	#print(integrals)
	fourier_vals = np.sum(integrals, axis=0)
	#print(fourier_vals)
	bfi_ = bfi(fourier_vals[0]/tot_energy, fourier_vals[1]/tot_energy, fourier_vals[2]/tot_energy, fourier_vals[3]/tot_energy )
	#minkowsi p = 0, p= 1 and p =np.inf
	#min1 = minkowski_distance(spectral_data, 1)
	#min2 = minkowski_distance(spectral_data, 2)
	#minkowski_distance(spectral_data, np.inf)	
	#bfi_ = benjamin_feir_index(depth, max_freq, max_bandwidth, total_energy )
	#bfi_ = bfi(fourier_vals[0], fourier_vals[1], fourier_vals[2], fourier_vals[3] )
	#out = 0
	cross_v = cross_val(spectral_data)
	
	#spectral mean, median and standard deviation
	#if bfi > 1:
	#	out = 1
	out = bfi_
	returns = [station_name, out, hs,  ta,  total_energy,] # dmean_max]
	#returns.extend(spectral_data)
	returns.extend( np.mean(spectral_data, axis=0) )
	returns.extend( np.std(spectral_data, axis=0) )
	returns.extend( np.median(spectral_data, axis=0) )
	#returns.extend(np.ndarray.flatten(np.array(dft_vals) ) )
	returns.extend(np.ndarray.flatten(np.array(cov_corr) ) )
	#returns.extend(min1)
	#returns.extend(min2)
	returns.extend(np.ndarray.flatten(np.array(cross_v) ) )
	#returns.extend(np.mean(cross_v, axis=1) )
	#returns.extend(np.std(cross_v, axis=1) )
	#returns.extend(np.median(cross_v, axis=1) )
	#add energy vals
	returns.append( np.std(energies ) )
	returns.append( np.mean(energies ) )
	returns.append( np.std(energies) )
	#print(returns)
	return returns


file_out = "training_data_all_spectral_vals.txt" 

def gen_data(filename):
	to_write = []
	#for filename in paths:
	temps = []
	with open(filename, "r") as f:
		with open("dataset/" + filename.split("/")[-1], "w") as f1:
			current_data = ""
			for line in f.readlines():
				if line[:5] == "File ":
					temp = copy.deepcopy(current_data)
					data = [x for x in temp.split("\n")]
					#process the data
					#print(data)
					#try:
					if data != ['']:
						vals = process_data(data)
						temps.append(vals)
					current_data = ""
				#except Exception as e:
				#	print(e)
				#current_data = ""
				else:
					current_data += line
	#to_write.extend(temps)
	return temps
#print(paths)
#to_write = []
#for a in range(10):
#	to_write_ = Parallel(n_jobs=-1) (delayed(gen_data) (i) for i in paths[ int(len(paths)/10) * a :int(len(paths)/10) * (a+1) ]  )
#	to_write.extend(to_write_)
#	del to_write_
#temp =
#to_write_ = Parallel(n_jobs=-1) (delayed(gen_data ) (i) for i in paths[:200] )
def split_array(arr):
	returns = []
	indices = []
	#remove indices with [0] ie no use
	my_arr = []
	for val in arr:
		if isinstance(val, list):#val == [0]:
			#continue
			#else:
			my_arr.append(val )
	
	arr = my_arr
	stations = {}
	
	for val in range(0, len(arr) - 1 ):
		#print( arr[val][0] )
		#print("==================")
		#if arr[val][0] != arr[val+1][0]:
		#	indices.append(val+1)
		
		if arr[val][0] not in stations:
			stations[ arr[val][0] ] = [val]
		else:
			stations[ arr[val][0] ] = stations[ arr[val][0] ] + [val]
	#print(stations)
	for val in stations:
		if isinstance(val, str):
			returns.append([ arr[x] for x in stations[val] ] )
	#for val in range(len(indices) - 1):
	#	returns.append( arr[indices[val] : indices[val+1] ] )
	#print(returns )
	return returns  
#my_data = []
#with open(file_out, "w") as f:

def run(path):
	try:
		to_write = split_array( gen_data(path) )
		#print(to_write)
		#my_data = []
		for dat in to_write:
			#we need to split the dat for different stations probably at this point
			#first column is the station name
			#if len(dat) < 2: continue
			#print(dat)
			#try:
			all_vals = split_array(dat)
			my_data = []
			for val in all_vals:
				outputs = [ [x[1] ]  for x in val ][1:]
				inputs = [x[2:] for x in val ][:len(val) - 1 ]
				#my_data = []
				for x in range(len(outputs)):
					temp = outputs[x] + inputs[x]
					my_data.append(temp)
			#print(my_data)
			file_out = path.split("/")[-1]
			with open("dataset/" + file_out, "a") as f:
				for values in my_data:
					if len(values) == len(my_data[1] ) :
						f.write(",".join([str(x) for x in values] )  + "\n")
	except Exception as e:
		print(e)
		print("invalid value")
to_write_ = Parallel(n_jobs=-1) (delayed(run ) (i) for i in paths )

#random.shuffle(my_data)
#with open(file_out, "w") as f:
#	for values in my_data[:int(len(my_data)/2) ]:
#		#if values[0]!= 0 and values[1] != 0:
#		if len(values) == len(to_write[0][0] ) :
#			f.write(",".join([str(x) for x in values] )  + "\n")
#with open("testing_data_spectral_vals.txt", "w") as f:
#	for values in my_data[int(len(my_data)/2):]:
#		if len(values) == len(to_write[0][0] ) :
#			f.write(",".join([str(x) for x in values] )  + "\n")
