from pathlib import Path
import multiprocessing as mp

from joblib import Parallel, delayed
from features import *
import numpy as np

import random
import math
from scipy.optimize import fsolve
import sys
import os
import copy
from bfi import *

from itertools import islice
random.seed(200)

def get_filepaths(directory):
	file_paths = []  # List which will store all of the full filepaths.

	# Walk the tree.
	for root, directories, files in os.walk(directory):
		for filename in files:
			# Join the two strings in order to form the full filepath.
			filepath = os.path.join(root, filename)
			file_paths.append(filepath)  # Add it to the list.
	return file_paths  # Self-explanatory.

# Run the above function and store its results in a variable.   
full_file_paths = get_filepaths("nodc/")

#print([ x for x in full_file_paths if x.split("/")[-1][:2] =="sp" ])


paths = [ x for x in full_file_paths if x.split("/")[-1][:2] =="sp" ]#[20:25]

from collections import deque

def window(seq, n=2):
	it = iter(seq)
	win = deque((next(it, None) for _ in range(n)), maxlen=n)	
	yield win
	append = win.append
	for e in it:
		append(e)
		yield win

def process_data(mylist):
	station_name = " ".join([ val for val in mylist[0].split(" ") if val.strip()!="" ][2:6] )	
	#hs = float(other_vars[1])
	#tp = float(other_vars[3])
	other_vars = [x for x in mylist[5].split(" ") if x.strip()!=""]
	hs = float(other_vars[1])
	tp = float(other_vars[3])
	#dp = float(other_vars[5])
	ta = float(other_vars[7])
	depth = [ x for x in mylist[3].split(" ") if x.strip()!=''][2]
	spec_data = [ x for x in mylist[10:] if x!='' and x!= [] ]
	frequencies = []
	spectral_data = []
	bandwidths = []
	energies = []
	total_energy = 0
	outs = []
	#print(spec_data)
	for spec in spec_data:
		dat = [x for x in spec.split(" ") if x.strip()!="" ]
		#try:

		frequencies.append( float(dat[0] )  )
		spectral_data.append([ float(x) for x in dat[2: 8] ] )
		#outs.append([ float(x) * energy * float(dat[1] ) for x in dat[4: 8] ] )
		try:
			energy = float(dat[2])
		except Exception as e:
			energy = 0.000001
			return [0] 
		outs.append([ float(x) * energy * float(dat[1] ) for x in dat[4: 8] ] )
		energies.append(energy)
		bandwidths.append( float(dat[1] ) )
		total_energy += float(dat[1] ) * energy															
		#except Exception as e:
		#	print("hello")
		#	return {'station': station_name, }						
	return_val = {}
	return_val['station'] = station_name
	return_val['hs'] = hs
	return_val['tp'] = tp
	return_val['ta'] = ta
	return_val['outs'] = outs
	return_val['freq'] = frequencies
	return_val['spectra'] = spectral_data
	return_val['energies'] = energies
	return_val['bandwidth'] = bandwidths
	return_val['total_energy'] = total_energy
	#print(return_val)
	return return_val 
def gen_data(filename, dirpath):
	to_write = []
        #for filename in paths:
	temps = []
	with open(filename, "r") as f:
		with open(dirpath + filename.split("/")[-1], "w") as f1:
			current_data = ""
			for line in f.readlines():
				if line[:5] == "File ":
					temp = copy.deepcopy(current_data)
					data = [x for x in temp.split("\n")]
                                        #process the data
                                        #print(data)
					try:
						if data != ['']:
							vals = process_data(data)
							temps.append(vals)
							current_data = ""
					except Exception as e:
						print(e)
						print(filename )
                                #current_data = ""
				else:
					current_data += line
        #to_write.extend(temps)
	return temps
#first open the files




# then find the data by stations and sort it 
def split_array(arr):
	returns = []
        #indices = []
        #remove indices with [0] ie no use
	my_arr = []
	for val in arr:
		if isinstance(val, dict):#val == [0]:
                        #continue
                        #else:
			my_arr.append(val )

	arr = my_arr
	stations = {}
	#print(arr)
	for val in range(0, len(arr) - 1 ):
                #print( arr[val][0] )
                #print("==================")
                #if arr[val][0] != arr[val+1][0]:
                #       indices.append(val+1)
		#print(arr[val])
		if arr[val]['station'] not in stations:
                        stations[ arr[val]['station'] ] = [val]
		else:
			stations[ arr[val]['station'] ] = stations[ arr[val][ 'station' ] ] + [val]
        #print(stations)
	for val in stations:
		if isinstance(val, str):
			returns.append([ arr[x] for x in stations[val] ] )
        #for val in range(len(indices) - 1):
        #       returns.append( arr[indices[val] : indices[val+1] ] )
        #print(returns )
	return returns


#calculate spectra and other features

print(paths[200])

#to_write =  split_array( gen_data(paths[200]) )

#to_write = gen_data(paths[200] )
def run(path, window_size=2, dirpath='dataset/', step = 1 ):
	#step = window_size - 1
	to_write = window(gen_data(path, dirpath ), window_size  )
	X = []
	y = []
	with open(dirpath + path.split("/")[-1] , "w") as f:
		for x in to_write :
			#print(x)
			try: #to handle inconsistencies and remove data that is not complete
				#now, we add the other elements to the array but multiply the fourier values
				first_val = [ x[0]['hs'], x[0]['tp'], x[0]['ta'], x[0]['total_energy'] ]
				bandwidth = x[0]['bandwidth']
				spectra = np.array( x[0]['spectra'] )	
				energies = np.array( x[0]['energies'] )
				#total_energy = x[0]['total_energy']
				#do pairwise multiplication of the spectra values
				pool = mp.Pool(mp.cpu_count() )
				for val in list(x)[1: ]:
					first_val.extend( [ val['hs'], val['tp'], val['ta'], val['total_energy']]  )
					spectra = np.multiply(spectra, np.array(val['spectra']) )
					energies = np.multiply(energies, np.array(val['energies']) )
				#result = pool.map(np.multiply(spectra, np.array(spectra) )
				#pass
				cov_corr = covariance_and_correlation(spectra)
				#print(integrals)
				fourier_vals = np.sum(spectra, axis=0)
				
				cross_v = cross_val(spectra)
				#first_val.extend([ item for sublist in spectra for item in sublist ] )
				first_val.extend(np.ndarray.flatten(cov_corr))			
				first_val.extend(np.ndarray.flatten(cross_v))
				first_val.extend(np.ndarray.flatten(fourier_vals))
				#first_val.extend([ item for sublist in energies for item in sublist ] )
				#first_val.extend(energies)
				#first_val = np.ndarray.flatten(np.array(first_val) )
				#calculate y from the last element of the array
				first_val.append( np.std(energies ) )
				first_val.append( np.mean(energies ) )
				first_val.append( np.std(energies) )	
				#print(first_val)
				last_vals = list(x)[-1]
				fourier_vals = np.sum(last_vals['outs'], axis=0)
				tot_energy = last_vals['total_energy']
				#print(fourier_vals)		
				#if len(fourier_vals) == 4:
				#print(fourier_vals)
				y_val = bfi(fourier_vals[0]/tot_energy, fourier_vals[1]/tot_energy, fourier_vals[2]/tot_energy, fourier_vals[3]/tot_energy )			
				X.append( np.ndarray.flatten(np.array(first_val) ) ) 
				y.append(y_val)	
				#print("\n")
			except Exception as e:
				#write the data till now in a file
				X_forward = X[:len(X) - step]
				y_forward = y[step:]
				my_data = []
				for a in range(len(X_forward)):
					val = [ y_forward[a] ]
					val.extend(X_forward[a] )
					my_data.append(val)
				for values in my_data:
					if len(values) == len(my_data[1] ) :
						f.write(",".join([str(x) for x in values] )  + "\n")
				X = []
				y = []
		X_forward = X[:len(X) - step]
		y_forward = y[step:]
		my_data = []
		for a in range(len(X_forward)):
			val = [ y_forward[a] ]
			val.extend(X_forward[a] )
			my_data.append(val)
		#with open(dirpath + path.split("/")[-1] , "w") as f:
		for values in my_data:
			if len(values) == len(my_data[1] ) :
				f.write(",".join([str(x) for x in values] )  + "\n")		 	
	#return X, y


step = 1
window_s = 2
import os
if not os.path.exists("step" + str(step) + "/window_" + str(window_s) + "_dataset" ):
	Path( "step" + str(step) + "/window_" + str(window_s) + "_dataset").mkdir(parents=True, exist_ok=True)
Parallel(n_jobs=-1) (delayed(run ) (i, window_size=window_s, dirpath="step" + str(step) + "/window_" + str(window_s) + "_dataset" + '/', step = step) for i in paths )
#print(to_write)
